<?php

namespace Drupal\measurement_protocol;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;

/**
 * Manages tracking of document-related pages.
 */
class DocumentTracker {

  /**
   * Name of HTTP header variable used to denote trackable files.
   */
  const HEADER_NAME_SCHEME = 'Trackable-download-scheme';

  /**
   * The Google measurement protocol tracker service.
   *
   * @var \Drupal\measurement_protocol\Tracker
   */
  protected $tracker;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The stream wrapper manager.
   *
   * @var \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface
   */
  protected $streamWrapperManager;

  /**
   * Class constructor.
   *
   * @param \Drupal\measurement_protocol\Tracker $tracker
   *   The Google measurement protocol tracker service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface $stream_wrapper_manager
   *   The stream wrapper manager.
   */
  public function __construct(Tracker $tracker,
                              ConfigFactoryInterface $config_factory,
                              EntityTypeManagerInterface $entity_type_manager,
                              StreamWrapperManagerInterface $stream_wrapper_manager) {
    $this->tracker = $tracker;
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->streamWrapperManager = $stream_wrapper_manager;
  }

  /**
   * Prepare information about file download and send to Google Analytics.
   *
   * @param string $uri
   *   URI of a file, with stream wrapper as protocol.
   */
  public function trackDownload($uri) {
    $settings = $this->configFactory->get('measurement_protocol.settings');
    if (!$settings->get('documents.enabled')) {
      return;
    }
    if (!$files = $this->entityTypeManager->getStorage('file')->loadByProperties(['uri' => $uri])) {
      return;
    }
    $file = current($files);
    // Prepare all data related to the download of this file.
    switch ($hit_type = $this->tracker->getHitType('documents')) {
      case Tracker::HIT_TYPE_PAGEVIEW:
        $data = [
          'path' => file_url_transform_relative(file_create_url($uri)),
          'title' => 'Download file ' . $file->getFilename(),
        ];
        break;

      case Tracker::HIT_TYPE_EVENT:
        $data = [
          'category' => 'document',
          'action' => 'download',
          'id' => $file->id(),
          'label' => $file->getFilename(),
        ];
        break;
    }
    // @todo add a hook to allow modules to alter sent data.
    // Send data to Google Analytics:
    $this->tracker->track($hit_type, $data);
  }

  /**
   * Adds HTTP headers (temporarily) to identify trackable files.
   *
   * @param string $uri
   *   URI of a file, with stream wrapper as protocol.
   *
   * @return array|null
   *   Associative array containing the header variables needed, or NULL.
   *
   * @see measurement_protocol_file_download()
   */
  public function addHeaders($uri) {
    // If document tracking is disabled or document isn't private,
    // don't add any headers.
    $settings = $this->configFactory->get('measurement_protocol.settings');
    if (!$settings->get('documents.enabled')) {
      return;
    }
    $scheme = $this->streamWrapperManager->getScheme($uri);
    if ($scheme != 'private') {
      return;
    }
    // If the file download should be tracked, add the appropriate header.
    // This header variable will be removed later, but is needed to preserve
    // the information in ResponseSubscriber::onResponse().
    return [
      static::HEADER_NAME_SCHEME => $scheme,
    ];
  }

}
