<?php

namespace Drupal\measurement_protocol\EventSubscriber;

use Drupal\Core\Routing\RouteMatch;
use Drupal\measurement_protocol\DocumentTracker;
use Drupal\measurement_protocol\UserTracker;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\Routing\Matcher\UrlMatcherInterface;

/**
 * Event subscriber which injects tracking mechanism to supported pages.
 */
class ResponseSubscriber implements EventSubscriberInterface {

  /**
   * The Google measurement protocol tracker service.
   *
   * @var \Drupal\measurement_protocol\DocumentTracker
   */
  protected $documentTracker;

  /**
   * The Google measurement protocol tracker service.
   *
   * @var \Drupal\measurement_protocol\UserTracker
   */
  protected $userTracker;

  /**
   * Symfony's URL matcher..
   *
   * @var \Symfony\Component\Routing\Matcher\UrlMatcherInterface
   */
  protected $router;

  /**
   * Class constructor.
   *
   * @param \Drupal\measurement_protocol\DocumentTracker $document_tracker
   *   The document Google measurement protocol tracker service.
   * @param \Drupal\measurement_protocol\UserTracker $user_tracker
   *   The user oogle measurement protocol tracker service.
   * @param \Symfony\Component\Routing\Matcher\UrlMatcherInterface $router
   *   Symfony's URL matcher.
   */
  public function __construct(DocumentTracker $document_tracker,
                              UserTracker $user_tracker,
                              UrlMatcherInterface $router) {
    $this->documentTracker = $document_tracker;
    $this->userTracker = $user_tracker;
    $this->router = $router;
  }

  /**
   * Triggers GA tracking of supported HTTP responses.
   *
   * @param \Symfony\Component\HttpKernel\Event\FilterResponseEvent $event
   *   The response event.
   */
  public function onResponse(FilterResponseEvent $event) {
    $response = $event->getResponse();

    // Tracking of user-related actions.
    if ($response instanceof RedirectResponse) {
      $request = $event->getRequest();
      $route_from = RouteMatch::createFromRequest($request);
      switch ($route_from->getRouteName()) {
        // From 'user.reset' to 'user.reset.form' means success.
        // From 'user.reset' to 'user.reset' means it will be repeated.
        // From 'user.reset' to any other route means failure.
        case 'user.reset':
          $uri = $request->getRequestUri();
          // Determine the destination route from the absolute destination URL.
          $route_to = $this->router->match($response->getTargetUrl());
          $success = $route_to['_route'] == 'user.reset.form';
          // Pass all the parameters of the request to Tracker service.
          $parameters = $route_from->getParameters()->all();
          $this->userTracker->trackUserPasswordReset($uri, $success, $parameters);
          break;
      }
    }
    // Tracking of private file downloads.
    elseif ($response instanceof BinaryFileResponse) {
      // Check if custom header, which means that download should be tracked
      // in GA, was set.
      $scheme = $response->headers->get(DocumentTracker::HEADER_NAME_SCHEME);
      if (!empty($scheme)) {
        $file = $response->getFile();
        if ($file instanceof \SplFileInfo) {
          $uri = $file->getPathName();
          $response->headers->remove(DocumentTracker::HEADER_NAME_SCHEME);
          $this->documentTracker->trackDownload($uri);
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::RESPONSE][] = ['onResponse'];
    return $events;
  }

}
