<?php

namespace Drupal\measurement_protocol\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\measurement_protocol\Tracker;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Google_Analytics settings for this site.
 */
class AdminSettingsForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'measurement_protocol.settings';

  /**
   * The stream wrapper manager.
   *
   * @var \Drupal\measurement_protocol\Tracker
   */
  protected $tracker;

  /**
   * {@inheritdoc}
   */
  public function __construct(ConfigFactoryInterface $config_factory,
                              Tracker $tracker) {
    parent::__construct($config_factory);
    $this->tracker = $tracker;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      // Load the service required to construct this class.
      $container->get('config.factory'),
      $container->get('measurement_protocol.tracker')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'measurement_protocol_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [static::SETTINGS];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);

    $form['#tree'] = TRUE;

    $form['documents'] = [
      '#type' => 'details',
      '#title' => $this->t('Documents'),
      '#open' => TRUE,
    ];
    $form['documents']['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $config->get('documents.enabled'),
    ];
    $form['documents']['hit_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Hit type'),
      '#options' => $this->getHitTypeOptions(),
      '#default_value' => $config->get('documents.hit_type'),
      '#states' => [
        'visible' => [
          ':input[name="documents[enabled]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['user_password_reset'] = [
      '#type' => 'details',
      '#title' => $this->t('User password reset'),
      '#open' => TRUE,
    ];
    $form['user_password_reset']['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $config->get('user_password_reset.enabled'),
    ];
    $form['user_password_reset']['hit_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Hit type'),
      '#options' => $this->getHitTypeOptions(),
      '#default_value' => $config->get('user_password_reset.hit_type'),
      '#states' => [
        'visible' => [
          ':input[name="user_password_reset[enabled]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['advanced'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced settings'),
      '#open' => FALSE,
    ];
    $form['advanced']['random_anonymous_client_id'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Generate random UUID for anonymous Google Analytics client id'),
      '#description' => $this->t('Otherwise, fixed value "555" will be used.'),
      '#default_value' => $config->get('random_anonymous_client_id'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Get the options for 'Hit type' radios field.
   *
   * @return array
   *   Array where key is one of HIT_TYPE_X constanct from this class,
   *   and value is a descriptive label.
   */
  protected function getHitTypeOptions() {
    return $this->tracker->getSupportedHitTypes();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $this->configFactory->getEditable(static::SETTINGS)
      ->set('documents', $form_state->getValue('documents'))
      ->set('user_password_reset', $form_state->getValue('user_password_reset'))
      ->set('random_anonymous_client_id', $form_state->getValue('random_anonymous_client_id'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
