<?php

namespace Drupal\measurement_protocol;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\HttpFoundation\RequestStack;
use TheIconic\Tracking\GoogleAnalytics\Analytics;

/**
 * Manages general GA tracking tasks.
 */
class Tracker {

  use StringTranslationTrait;

  /**
   * Value sent when tracking a page view by a user.
   */
  const HIT_TYPE_PAGEVIEW = 'pageview';

  /**
   * Value sent when tracking a user event.
   */
  const HIT_TYPE_EVENT = 'event';

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger channel from "Google analytics" module.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The main object for client interaction from the PHP library.
   *
   * @var \TheIconic\Tracking\GoogleAnalytics\Analytics
   */
  protected $analytics;

  /**
   * Class constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack used to retrieve the current request.
   */
  public function __construct(ConfigFactoryInterface $config_factory,
                              LoggerChannelFactoryInterface $logger_factory,
                              RequestStack $request_stack) {
    $this->configFactory = $config_factory;
    $this->logger = $logger_factory->get('google_analytics');
    $this->requestStack = $request_stack;
  }

  /**
   * Send request to Google Analytics.
   *
   * @param string $hit_type
   *   One of HIT_TYPE_X constanct from this class.
   * @param array $data
   *   Array containing data needed for this hit type in Google analytics.
   */
  public function track($hit_type, array $data) {
    try {
      // If there is something wrong with Analytics setup, do nothing.
      if (!$this->initializeAnalytics()) {
        return;
      }
      switch ($hit_type) {
        case static::HIT_TYPE_PAGEVIEW:
          $this->trackPageView($data);
          break;

        case static::HIT_TYPE_EVENT:
          $this->trackEvent($data);
          break;
      }
    }
    // Make sure to catch any exception (ex. from GuzzleHTTP, or Analytics
    // class) but this will not interrupt page execution.
    catch (\Exception $e) {
      $this->logError($e);
    }
  }

  /**
   * Initializes the Analytics object with data common for all hit types.
   *
   * @return bool
   *   Whether the requirements are met and the initialization was successful.
   */
  protected function initializeAnalytics() {
    if (!$account = $this->getAccount()) {
      return FALSE;
    }
    // Instantiate the Analytics object. Optionally pass TRUE in the
    // constructor if you want to connect using HTTPS.
    $analytics = new Analytics(TRUE);
    $analytics
      ->setProtocolVersion('1')
      ->setTrackingId($account)
      ->setClientId($this->getClientId())
      ->setAnonymizeIp($this->isIpAnonymized());
    $this->analytics = $analytics;
    return TRUE;
  }

  /**
   * Send 'Page view' to Google Analytics.
   *
   * @param array $data
   *   An associative array containing:
   *     - path: The relative URL of the page, mandatory.
   *     - title: The title of the page, optional.
   *     - referrer: The URL which redirected to this page, optional.
   */
  protected function trackPageView(array $data) {
    if (empty($data['path'])) {
      return;
    }
    $this->analytics
      ->setDocumentPath($data['path']);
    if (!empty($data['title'])) {
      $this->analytics
        ->setDocumentTitle($data['title']);
    }
    if (!empty($data['referrer'])) {
      $this->analytics
        ->setDocumentReferrer($data['referrer']);
    }
    // @todo add any custom parameters if needed.
    $this->analytics->sendPageview();
  }

  /**
   * Send 'Event' to Google Analytics.
   *
   * @param array $data
   *   An associative array containing:
   *     - category: to be mapped to GA Event category.
   *     - action: to be mapped to GA Event action.
   *     - name: to be mapped to GA Event label.
   *     - id: The ID of Drupal File entity, to be mapped to GA Event value.
   */
  protected function trackEvent(array $data) {
    if (empty($data['category']) || empty($data['action'])) {
      return;
    }
    $this->analytics
      ->setEventCategory($data['category'])
      ->setEventAction($data['action']);
    if (!empty($data['id'])) {
      $this->analytics
        ->setEventValue($data['id']);
    }
    if (!empty($data['label'])) {
      $this->analytics
        ->setEventLabel($data['label']);
    }
    $this->analytics->sendEvent();
  }

  /**
   * Log the error which occurred communicating with GA.
   *
   * @param \Exception $e
   *   The exception object.
   */
  protected function logError(\Exception $e) {
    $this->logger->warning('An error occurred tracking the resourse using Google Analytics Management protocol: (@code) %message', [
      '@code' => $e->getCode(),
      '%message' => $e->getMessage(),
    ]);
  }

  /**
   * Get Google Analytics account.
   *
   * @return string|null
   *   The Google Analytics account, in the form of UA-xxxxxxx-yy,
   *   or NULL if no GA account is set or it is invalid.
   */
  protected function getAccount() {
    $account = $this->configFactory->get('google_analytics.settings')->get('account');
    return $account && preg_match('/^UA-\d+-\d+$/', $account) ? $account : NULL;
  }

  /**
   * Check if user IP should be anonymized (ex. for privacy reasons).
   *
   * @return bool
   *   Whether GA should send full or partial user IP (as set in GA module).
   */
  protected function isIpAnonymized() {
    return $this->configFactory->get('google_analytics.settings')->get('privacy.anonymizeip');
  }

  /**
   * Returns a client ID to be used for the cid parameter.
   *
   * If the '_ga' cookie is available then use it.  If it is not available,
   * then we generate our own UUID or pass the anonymous CID.
   *
   * @return string
   *   An ID suitable for the cid parameter.
   */
  protected function getClientId() {
    // If GA cookie exists (ie. user has already visited regular pages of
    // this domain), send the same information as it would be sent via JS.
    // The cookie example: "GA1.1.904941809.1556093647".
    $ga_cookie = $this->requestStack->getCurrentRequest()->cookies->get('_ga');
    if (!empty($ga_cookie)) {
      $_ga_parts = explode('.', $ga_cookie);
      return $_ga_parts[2] . '.' . $_ga_parts[3];
    }
    // It is possible that the user has opened the download link in browser
    // without any prior regular visits to this domain (ie. no GA cookie).
    // In this case return the Google Analytics anonymous client ID.
    else {
      if (!$this->configFactory->get('measurement_protocol.settings')->get('random_anonymous_client_id')) {
        return '555';
      }
      else {
        return $this->generateUuid();
      }
    }
  }

  /**
   * Generates a UUID.
   *
   * @return string
   *   A string containing a UUID.
   *
   * @see http://www.php.net/manual/en/function.uniqid.php#94959
   */
  protected function generateUuid() {
    return sprintf('%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
      // 32 bits for "time_low".
      mt_rand(0, 0xffff), mt_rand(0, 0xffff),
      // 16 bits for "time_mid".
      mt_rand(0, 0xffff),
      // 16 bits for "time_hi_and_version".
      // four most significant bits holds version number 4.
      mt_rand(0, 0x0fff) | 0x4000,
      // 16 bits, 8 bits for "clk_seq_hi_res".
      // 8 bits for "clk_seq_low".
      // two most significant bits holds zero and one for variant DCE1.1.
      mt_rand(0, 0x3fff) | 0x8000,
      // 48 bits for "node".
      mt_rand(0, 0xffff), mt_rand(0, 0xffff), mt_rand(0, 0xffff)
    );
  }

  /**
   * Get all Google Analytics hit types that this module supports.
   *
   * @return array
   *   Array where key is one of HIT_TYPE_X constant from this class,
   *   and value is a descriptive label.
   */
  public function getSupportedHitTypes() {
    return [
      static::HIT_TYPE_PAGEVIEW => $this->t('Page view'),
      static::HIT_TYPE_EVENT => $this->t('Event'),
    ];
  }

  /**
   * Get Google Analytics hit type configured for this tracking type.
   *
   * @param string $track_type
   *   Tracking type that this module supports, ex. document download.
   *
   * @return string|bool
   *   Array where key is one of HIT_TYPE_X constanct from this class,
   *   and value is a descriptive label.
   */
  public function getHitType($track_type) {
    $settings = $this->configFactory->get('measurement_protocol.settings')->get($track_type);
    if (empty($settings)) {
      return FALSE;
    }
    return $settings['hit_type'] ?? static::HIT_TYPE_PAGEVIEW;
  }

}
